@@ -45,8 +45,9 @@ static gboolean zenity_general_dialog_no_markup;
 static gint     zenity_general_timeout_delay;
 static gchar   *zenity_general_ok_button;
 static gchar   *zenity_general_cancel_button;
+static gchar  **zenity_general_extra_buttons;
 static gboolean zenity_general_modal;
-static gint     zenity_general_attach;
+static guintptr zenity_general_attach;
 static gboolean zenity_general_dialog_ellipsize;
 
 /* Calendar Dialog Options */
@@ -104,6 +105,7 @@ static gboolean zenity_progress_time_remaining;
 /* Question Dialog Options */
 static gboolean zenity_question_active;
 static gboolean zenity_question_default_cancel;
+static gboolean zenity_question_switch;
 
 /* Text Dialog Options */
 static gboolean zenity_text_active;
@@ -210,7 +212,7 @@ static GOptionEntry general_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_STRING,
     &zenity_general_ok_button,
-    N_("Sets the label of the Ok button"),
+    N_("Set the label of the OK button"),
     N_("TEXT")
   },
   {
@@ -219,7 +221,16 @@ static GOptionEntry general_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_STRING,
     &zenity_general_cancel_button,
-    N_("Sets the label of the Cancel button"),
+    N_("Set the label of the Cancel button"),
+    N_("TEXT")
+  },
+  {
+    "extra-button",
+    '\0',
+    0,
+    G_OPTION_ARG_STRING_ARRAY,
+    &zenity_general_extra_buttons,
+    N_("Add an extra button"),
     N_("TEXT")
   },
   {
@@ -391,7 +402,7 @@ static GOptionEntry error_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_NONE,
     &zenity_general_dialog_no_markup,
-    N_("Do not enable pango markup")
+    N_("Do not enable Pango markup")
   },
   {
 	"ellipsize",
@@ -449,7 +460,7 @@ static GOptionEntry info_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_NONE,
     &zenity_general_dialog_no_markup,
-    N_("Do not enable pango markup")
+    N_("Do not enable Pango markup")
   },
   {
 	"ellipsize",
@@ -534,7 +545,7 @@ static GOptionEntry file_selection_options[] = {
     0,
     G_OPTION_ARG_STRING_ARRAY,
     &zenity_file_filter,
-    N_("Sets a filename filter"),
+    N_("Set a filename filter"),
     /* Help for file-filter argument (name and patterns for file selection) */
     N_("NAME | PATTERN1 PATTERN2 ..."),
   },
@@ -577,7 +588,7 @@ static GOptionEntry list_options[] = {
     0,
     G_OPTION_ARG_NONE,
     &zenity_list_checklist,
-    N_("Use check boxes for first column"),
+    N_("Use check boxes for the first column"),
     NULL
   },
   {
@@ -586,7 +597,7 @@ static GOptionEntry list_options[] = {
     0,
     G_OPTION_ARG_NONE,
     &zenity_list_radiolist,
-    N_("Use radio buttons for first column"),
+    N_("Use radio buttons for the first column"),
     NULL
   },
   {
@@ -595,7 +606,7 @@ static GOptionEntry list_options[] = {
     0,
     G_OPTION_ARG_NONE,
     &zenity_list_imagelist,
-    N_("Use an image for first column"),
+    N_("Use an image for the first column"),
     NULL
   },
   {
@@ -650,7 +661,7 @@ static GOptionEntry list_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_NONE,
     &zenity_list_hide_header,
-    N_("Hides the column headers"),
+    N_("Hide the column headers"),
     NULL
   },
   {
@@ -765,7 +776,6 @@ static GOptionEntry progress_options[] = {
     0,
     G_OPTION_ARG_NONE,
     &zenity_progress_auto_kill,
-    /* xgettext: no-c-format */
     N_("Kill parent process if Cancel button is pressed"),
     NULL
   },
@@ -775,7 +785,6 @@ static GOptionEntry progress_options[] = {
    0,
    G_OPTION_ARG_NONE,
    &zenity_progress_no_cancel,
-   /* xgettext: no-c-format */
    N_("Hide Cancel button"),
    NULL
   },
@@ -785,6 +794,7 @@ static GOptionEntry progress_options[] = {
     0,
     G_OPTION_ARG_NONE,
     &zenity_progress_time_remaining,
+    /* xgettext: no-c-format */
     N_("Estimate when progress will reach 100%"),
     NULL
   },
@@ -836,7 +846,7 @@ static GOptionEntry question_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_NONE,
     &zenity_general_dialog_no_markup,
-    N_("Do not enable pango markup")
+    N_("Do not enable Pango markup")
   },
   {
     "default-cancel",
@@ -844,7 +854,7 @@ static GOptionEntry question_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_NONE,
     &zenity_question_default_cancel,
-    N_("Give cancel button focus by default"),
+    N_("Give Cancel button focus by default"),
     NULL
   },
   {
@@ -855,6 +865,15 @@ static GOptionEntry question_options[] = {
 	&zenity_general_dialog_ellipsize,
 	N_("Enable ellipsizing in the dialog text. This fixes the high window size with long texts")
   },
+  {
+    "switch",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_question_switch,
+    N_("Suppress OK and Cancel buttons"),
+    NULL
+  },
   { 
     NULL 
   }
@@ -913,7 +932,7 @@ static GOptionEntry text_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_NONE,
     &zenity_text_enable_html,
-    N_("Enable html support"),
+    N_("Enable HTML support"),
     NULL
   },
   {
@@ -931,7 +950,7 @@ static GOptionEntry text_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_STRING,
     &zenity_text_url,
-    N_("Sets an url instead of a file. Only works if you use --html option"),
+    N_("Set an URL instead of a file. Only works if you use --html option"),
     N_("URL")
   },
 #endif
@@ -992,7 +1011,7 @@ static GOptionEntry warning_options[] = {
     G_OPTION_FLAG_NOALIAS,
     G_OPTION_ARG_NONE,
     &zenity_general_dialog_no_markup,
-    N_("Do not enable pango markup")
+    N_("Do not enable Pango markup")
   },
   {
 	"ellipsize",
@@ -1341,7 +1360,9 @@ zenity_option_free (void) {
     g_free (zenity_general_ok_button);
   if (zenity_general_cancel_button)
     g_free (zenity_general_cancel_button);
-
+  if (zenity_general_extra_buttons)
+    g_strfreev (zenity_general_extra_buttons);
+    
   if (zenity_calendar_date_format)
     g_free (zenity_calendar_date_format);
 
@@ -1463,6 +1484,7 @@ zenity_general_pre_callback (GOptionContext *context,
   zenity_general_uri = NULL;
   zenity_general_ok_button = NULL;
   zenity_general_cancel_button = NULL;
+  zenity_general_extra_buttons = NULL;
   zenity_general_dialog_no_wrap = FALSE;
   zenity_general_dialog_no_markup = FALSE;
   zenity_general_timeout_delay = -1;
@@ -1594,7 +1616,7 @@ zenity_question_pre_callback (GOptionContext *context,
 {
   zenity_question_active = FALSE;
   zenity_question_default_cancel = FALSE;
-
+  zenity_question_switch = FALSE;
   return TRUE;
 }
 
@@ -1710,6 +1732,7 @@ zenity_general_post_callback (GOptionContext *context,
   results->data->timeout_delay = zenity_general_timeout_delay;
   results->data->ok_label = zenity_general_ok_button;
   results->data->cancel_label = zenity_general_cancel_button;
+  results->data->extra_label = zenity_general_extra_buttons;
   results->data->modal = zenity_general_modal;
   results->data->attach = zenity_general_attach;
 
@@ -2012,13 +2035,18 @@ zenity_question_post_callback (GOptionContext *context,
   if (results->mode == MODE_QUESTION) {
     results->msg_data->dialog_text = zenity_general_dialog_text;
     results->msg_data->dialog_icon = zenity_general_dialog_icon;
-    results->msg_data->mode = ZENITY_MSG_QUESTION;
+    if(zenity_question_switch)
+      results->msg_data->mode = ZENITY_MSG_SWITCH;
+    else
+      results->msg_data->mode = ZENITY_MSG_QUESTION;
     results->msg_data->no_wrap = zenity_general_dialog_no_wrap;
     results->msg_data->no_markup = zenity_general_dialog_no_markup;
-	results->msg_data->ellipsize = zenity_general_dialog_ellipsize;
+    results->msg_data->ellipsize = zenity_general_dialog_ellipsize;
     results->msg_data->default_cancel = zenity_question_default_cancel;
   }
-
+  if(zenity_question_switch && zenity_general_extra_buttons==NULL)
+    zenity_option_error (zenity_option_get_name (question_options, &zenity_question_switch), ERROR_SYNTAX);
+      
   return TRUE;
 }
 
@@ -2495,5 +2523,6 @@ zenity_option_parse (gint argc, gchar **argv)
     if (results->mode != MODE_INFO && results->mode != MODE_ERROR && results->mode != MODE_QUESTION && results->mode != MODE_WARNING)
       zenity_option_error (zenity_option_get_name (text_options, &zenity_general_dialog_ellipsize), ERROR_SUPPORT);
 
+
   return results; 
 }
