@@ -265,7 +265,7 @@ zenity_text (ZenityData *data, ZenityTextData *text_data)
 
   if (text_data->font) {
       PangoFontDescription *fontDesc = pango_font_description_from_string (text_data->font);
-      gtk_widget_modify_font (GTK_WIDGET(text_view), fontDesc);
+      gtk_widget_override_font (GTK_WIDGET(text_view), fontDesc);
   }
 
   if (text_data->uri)
@@ -276,16 +276,20 @@ zenity_text (ZenityData *data, ZenityTextData *text_data)
   if (text_data->editable)
     zen_text_data->buffer = text_buffer;
 
+  if (data->extra_label) {
+    gint i=0;
+    while(data->extra_label[i]!=NULL){
+      gtk_dialog_add_button (GTK_DIALOG (dialog), data->extra_label[i], i);
+      i++;
+    }
+  }
+
   if (data->ok_label) { 
     gtk_button_set_label (GTK_BUTTON (ok_button), data->ok_label);
-    gtk_button_set_image (GTK_BUTTON (ok_button),
-                          gtk_image_new_from_stock (GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON));
   }
 
   if (data->cancel_label) {
     gtk_button_set_label (GTK_BUTTON (cancel_button), data->cancel_label);
-    gtk_button_set_image (GTK_BUTTON (cancel_button), 
-                          gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON));
   }
 
   if (text_data->checkbox) {
@@ -398,7 +402,8 @@ zenity_text_dialog_response (GtkWidget *widget, int response, gpointer data)
       break;
 
     default:
-      /* Esc dialog */
+      if (response < g_strv_length(zen_data->extra_label))
+        printf("%s\n",zen_data->extra_label[response]);
       zenity_util_exit_code_with_data(ZENITY_ESC, zen_data);
       break;
   }
