@@ -305,8 +305,9 @@ zenity_mid_search_func (GtkTreeModel *model, gint column,
 {
     gchar *iter_string = NULL;
     gtk_tree_model_get (model, iter, column, &iter_string, -1); 
-    return  ! g_strrstr (g_utf8_strdown(iter_string, -1), 
-                         g_utf8_strdown(key, -1)) != NULL;
+    return  ! (g_strrstr (g_utf8_strdown(iter_string, -1),
+                          g_utf8_strdown(key, -1)) != NULL);
+
 }
 
 static void
@@ -396,18 +397,22 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
   if (data->modal)
     gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 
+  if (data->extra_label) {
+    gint i=0;
+    while(data->extra_label[i]!=NULL){
+      gtk_dialog_add_button (GTK_DIALOG (dialog), data->extra_label[i], i);
+      i++;
+    }
+  }
+
   if (data->ok_label) {
     button = GTK_WIDGET (gtk_builder_get_object (builder, "zenity_tree_ok_button"));
     gtk_button_set_label (GTK_BUTTON (button), data->ok_label);
-    gtk_button_set_image (GTK_BUTTON (button),
-                          gtk_image_new_from_stock (GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON));
   }
 
   if (data->cancel_label) {
     button = GTK_WIDGET (gtk_builder_get_object (builder, "zenity_tree_cancel_button"));
     gtk_button_set_label (GTK_BUTTON (button), data->cancel_label);
-    gtk_button_set_image (GTK_BUTTON (button),
-                          gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON));
   }
 
   text = gtk_builder_get_object (builder, "zenity_tree_text");
@@ -555,8 +560,6 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
     column_index++;
   }
 
-  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), TRUE);
-
   if (tree_data->hide_header)
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree_view), FALSE);
 
@@ -708,7 +711,8 @@ zenity_tree_dialog_response (GtkWidget *widget, int response, gpointer data)
       break;
 
     default:
-      /* Esc dialog */
+      if (response < g_strv_length(zen_data->extra_label))
+        printf("%s\n",zen_data->extra_label[response]);
       zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
       break;
   }
